# WRJoy - Project file
# Warthog Robotics
# University of Sao Paulo
# https://www.wr.sc.usp.br
# Copyright 2011-2020 Warthog Robotics
# This file is part of WRJoy project


# WRJoy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WRJoy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


# Application info
TEMPLATE = app
DESTDIR  = bin
TARGET   = WREye
VERSION  = 1.0.1
CONFIG  += c++11

# Qt config
CONFIG += console
CONFIG -= app_bundle
QT     += core gui widgets opengl

# Compilation flags
count(custom_cxxflags, 0) {
    QMAKE_CXXFLAGS_RELEASE = -O2 -Wfloat-equal -Wshadow -Woverloaded-virtual -Wold-style-cast

    count(arch, 1) {
        QMAKE_CXXFLAGS_RELEASE += -march=$$arch
    }
}

# Libraries
LIBS += -lGEARSystem -lomniORB4 -lomnithread -lprotobuf -lGLU


# Temporary dirs
OBJECTS_DIR = tmp/obj
MOC_DIR     = tmp/moc
UI_DIR      = tmp/ui
RCC_DIR     = tmp/rcc

# Project files
INCLUDEPATH *= . src 3rdparty

HEADERS     += src/WREye/entity/eyeclient/eyeclient.hh \
               src/WREye/entity/eyeupdater/eyeupdater.hh \
               src/WREye/entity/entity.hh \
               src/WREye/entity/eyebase.hh \
               src/WREye/object/ball/ball.hh \
               src/WREye/object/robot/robot.hh \
               src/WREye/object/object.hh \
               src/WREye/object/objectvelocity.hh \
               src/WREye/utils/color.hh \
               src/WREye/utils/wrtimer.hh \
               src/WREye/object/wrobjects.hh \
               src/WREye/exithandler.hh \
               src/WREye/utils/fieldtype.hh \
               src/WREye/entity/eyeupdater/robottracker.hh \
               src/WREye/GUI/maingui.h \
               src/WREye/GUI/soccerview/util/field.h \
               src/WREye/GUI/soccerview/util/field_default_constants.h \
               src/WREye/GUI/soccerview/util/geometry.h \
               src/WREye/GUI/soccerview/util/gvector.h \
               src/WREye/GUI/soccerview/util/timer.h \
               src/WREye/GUI/soccerview/gltext.h \
               src/WREye/GUI/soccerview/soccerview.h \
               src/WREye/entity/eyeupdater/multiobject/multiobject.hh \
               src/WREye/entity/eyeupdater/multiobject/multiobjectfilter.hh \
               src/WREye/object/filter/kalmanfilter2d/kalmanfilter2d.hh \
               src/WREye/object/filter/kalmanfilter2d/kalmanstate.hh \
               src/WREye/object/filter/kalmanfilter2d/matrix.hh \
               src/WREye/object/filter/lossfilter/lossfilter.hh \
               src/WREye/object/filter/filter.hh \
               src/WREye/object/filter/wrfilters.hh \
               src/WREye/utils/utils.hh \
               src/WREye/utils/fieldarea.hh \
               src/WREye/GUI/eyeviewguiupdater.hh \
               src/WREye/GUI/eyeviewgui.hh \
               src/WREye/GUI/eyeconfiggui.hh \
               src/WREye/object/filter/noisefilter/noisefilter.hh

SOURCES     += 3rdparty/robocup_ssl_client.cpp \
               3rdparty/netraw.cpp \
               3rdparty/messages_robocup_ssl_wrapper.pb.cc \
               3rdparty/messages_robocup_ssl_refbox_log.pb.cc \
               3rdparty/messages_robocup_ssl_geometry.pb.cc \
               3rdparty/messages_robocup_ssl_detection.pb.cc \
               src/WREye/entity/eyeclient/eyeclient.cc \
               src/WREye/entity/eyeupdater/eyeupdater.cc \
               src/WREye/entity/entity.cc \
               src/WREye/object/ball/ball.cc \
               src/WREye/object/robot/robot.cc \
               src/WREye/object/object.cc \
               src/WREye/object/objectvelocity.cc \
               src/WREye/main.cc \
               src/WREye/exithandler.cc \
               src/WREye/utils/fieldtype.cc \
               src/WREye/entity/eyeupdater/robottracker.cc \
               src/WREye/GUI/maingui.cpp \
               src/WREye/GUI/soccerview/util/field.cpp \
               src/WREye/GUI/soccerview/gltext.cpp \
               src/WREye/GUI/soccerview/soccerview.cpp \
               src/WREye/entity/eyeupdater/multiobject/multiobject.cc \
               src/WREye/entity/eyeupdater/multiobject/multiobjectfilter.cc \
               src/WREye/object/filter/kalmanfilter2d/kalmanfilter2d.cc \
               src/WREye/object/filter/kalmanfilter2d/kalmanstate.cc \
               src/WREye/object/filter/kalmanfilter2d/matrix.cc \
               src/WREye/object/filter/lossfilter/lossfilter.cc \
               src/WREye/object/filter/filter.cc \
               src/WREye/utils/fieldarea.cc \
               src/WREye/GUI/eyeconfiggui.cc \
               src/WREye/GUI/eyeviewgui.cc \
               src/WREye/GUI/eyeviewguiupdater.cc \
               src/WREye/object/filter/noisefilter/noisefilter.cc

OTHER_FILES += pre-build.sh

RESOURCES   += img/wreye.qrc


# Installation path
bin_files.files = bin/WREye

count(bin_path, 0) {
    bin_files.path = /usr/bin/
}

count(bin_path, 1) {
    bin_files.path = $$bin_path
}


INSTALLS += bin_files

