/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#ifndef EYECONFIG_HH
#define EYECONFIG_HH

#include <QtGui>
#include <QtCore>
#include <QtWidgets>
#include <WREye/entity/eyebase.hh>
#include <WREye/utils/fieldarea.hh>

namespace EyeGUI {

class EyeConfigGUI : public QWidget {
    Q_OBJECT
public:
    EyeConfigGUI(EyeClient *eyeClient, EyeUpdater *eyeUpdater, EyeViewGUI *eyeView, QWidget *parent = 0);
    ~EyeConfigGUI();
private:
    // Controlled systems
    EyeClient *_eyeClient;
    EyeUpdater *_eyeUpdater;
    EyeViewGUI *_eyeView;

    // Layout
    QVBoxLayout *_tabLayout;

    // General information
    QGroupBox *_groupInfo;
    QFormLayout *_groupInfo_layout;
    QLabel *_groupInfo_fieldType;
    QLabel *_groupInfo_portNumber;
    QLabel *_groupInfo_numCams;
    QLabel *_groupInfo_fpsDetection;
    QLabel *_groupInfo_fpsGeometry;

    // Field limits
    QGroupBox *_groupFLimits;
    QHBoxLayout *_groupFLimits_layout;
    QToolButton *_groupFLimits_btnQ1;
    QToolButton *_groupFLimits_btnQ2;
    QToolButton *_groupFLimits_btnQ3;
    QToolButton *_groupFLimits_btnQ4;

    // Configuration
    QGroupBox *_groupConfig;
    QFormLayout *_groupConfig_layout;
    QSpinBox *_groupConfig_portNumber;
    QComboBox *_groupConfig_fieldType;

    // GUI updating
    QTimer _updateTimer;

    // GUI
    void setupGUI();
    void loadValuesToGUI();
    void connectSignals();

    // Internal
    void fieldSideLimitBtnClicked(bool checked, FieldAreas::FieldArea fieldLimit);
    void setBtnChecked(QToolButton *btn, bool checked);
    QString _defBtnStylesheet;
private slots:
    void fieldSideLimitBtnQ1Clicked(bool checked);
    void fieldSideLimitBtnQ2Clicked(bool checked);
    void fieldSideLimitBtnQ3Clicked(bool checked);
    void fieldSideLimitBtnQ4Clicked(bool checked);
    void portNumberEdited();
    void fieldTypeChanged();

    void updateGUI();
};

}

#endif // EYECONFIG_HH
