/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#include "eyeviewgui.hh"
#include <WREye/GUI/eyeviewguiupdater.hh>
#include <WREye/GUI/soccerview/soccerview.h>

#define ENABLE_ICON ":/activate.png"
#define ENABLE_TEXT "Enable EyeView redraw"
#define DISABLE_ICON ":/deactivate.png"
#define DISABLE_TEXT "Disable EyeView redrawing"

using namespace EyeGUI;

EyeViewGUI::EyeViewGUI(FieldTypes::FieldType fieldType, EyeUpdater *eyeUpdater, QWidget *parent) : QWidget(parent) {
    _fieldType = fieldType;
    _eyeUpdater = eyeUpdater;

    // Sets the object name
    this->setObjectName(this->metaObject()->className());

    // Setups the GUI
    setupGUI();

    // Connects the signals
    connectSignals();

    // EyeViewUpdater thread
    _viewUpdater = new EyeViewUpdater(_eyeUpdater, _view);
    _viewUpdater->start();
}

EyeViewGUI::~EyeViewGUI() {
    _viewUpdater->stopRunning();
    _viewUpdater->wait();
    delete _viewUpdater;
}

void EyeViewGUI::setupGUI() {
    // Config layout
    _tabLayout = new QVBoxLayout();
    _tabLayout->setMargin(0);
    this->setLayout(_tabLayout);

    // Button on/off
    _btnOnOff = new QPushButton(QIcon(DISABLE_ICON), DISABLE_TEXT);
    _tabLayout->addWidget(_btnOnOff);

    // GLSoccerView
    _view = new GLSoccerView(_fieldType);
    _tabLayout->addWidget(_view);
}

void EyeViewGUI::connectSignals() {
    QObject::connect(_btnOnOff, SIGNAL(clicked()), this, SLOT(btnOnOffClicked()));
}

void EyeViewGUI::btnOnOffClicked() {
    _btnOnOff->setEnabled(false);

    if(_viewUpdater->isRunning()) { // Off
        // Stop running updater thread
        _viewUpdater->stopRunning();
        _viewUpdater->wait();

        // Clear detection view
        _view->clearDetectionView();

        // Update button text
        _btnOnOff->setText(ENABLE_TEXT);
        _btnOnOff->setIcon(QIcon(ENABLE_ICON));

    } else { // On
        // Start running updater thread
        _viewUpdater->restart();

        // Update button text
        _btnOnOff->setText(DISABLE_TEXT);
        _btnOnOff->setIcon(QIcon(DISABLE_ICON));
    }

    _btnOnOff->setEnabled(true);
}

void EyeViewGUI::setFieldType(FieldTypes::FieldType fieldType) {
    _fieldType = fieldType;
    _view->setFieldType(fieldType);
}
