/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#ifndef EYEVIEW_HH
#define EYEVIEW_HH

#include <QtGui>
#include <QtCore>
#include <QtWidgets>

#include <WREye/entity/eyebase.hh>
#include <WREye/utils/fieldtype.hh>

namespace EyeGUI {

class EyeViewGUI : public QWidget {
    Q_OBJECT
public:
    EyeViewGUI(FieldTypes::FieldType fieldType, EyeUpdater *eyeUpdater, QWidget *parent = 0);
    virtual ~EyeViewGUI();

    FieldTypes::FieldType fieldType() const { return _fieldType; }
    void setFieldType(FieldTypes::FieldType fieldType);

private:
    // Controlled objects
    EyeUpdater *_eyeUpdater;

    // Layout
    QVBoxLayout *_tabLayout;
    GLSoccerView *_view;
    QPushButton *_btnOnOff;

    // Internal
    FieldTypes::FieldType _fieldType;

    // EyeViewUpdater thread
    EyeViewUpdater *_viewUpdater;

    // GUI
    void setupGUI();
    void connectSignals();

private slots:
    void btnOnOffClicked();
};

}

#endif // EYEVIEW_HH
