/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#include "eyeviewguiupdater.hh"
#include <WREye/entity/eyeupdater/eyeupdater.hh>
#include <WREye/GUI/soccerview/soccerview.h>

using namespace EyeGUI;

QString EyeViewUpdater::name() {
    return "EyeViewUpdater";
}

EyeViewUpdater::EyeViewUpdater(EyeUpdater *eyeUpdater, GLSoccerView *view) {
    _eyeUpdater = eyeUpdater;
    _view = view;

    // Loop time, disabled
    this->setLoopTime(0);
}

EyeViewUpdater::~EyeViewUpdater() {

}

void EyeViewUpdater::initialization() {

}

void EyeViewUpdater::loop() {

    // Check if EyeUpdater has new detection updates
    if(_eyeUpdater->hasDetectionUpdates(VIEW_RESETTIME)) {

        // Get objects from EyeUpdater
        QList<Robot*> objRobots = _eyeUpdater->getRobotObjects();
        Ball *objBall = _eyeUpdater->getBallObject();

        // Update detection view
        _view->updateDetection(objRobots, objBall);

    } else {
        // Clear detection view
        _view->clearDetectionView();
    }


    // Check if EyeUpdater has new detection updates
    if(_eyeUpdater->hasGeometryUpdates()) {

        // Get field geometry from EyeUpdater
        SSL_GeometryFieldSize fieldGeometry = _eyeUpdater->getFieldGeometry();

        // Update detection view
        _view->updateFieldGeometry(fieldGeometry);

    }

}

void EyeViewUpdater::finalization() {

}
