/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#include "maingui.h"
#include <WREye/GUI/eyeconfiggui.hh>
#include <WREye/GUI/eyeviewgui.hh>
#include <WREye/entity/eyeclient/eyeclient.hh>
#include <WREye/entity/eyeupdater/eyeupdater.hh>

using namespace EyeGUI;

MainGUI::MainGUI(FieldTypes::FieldType fieldType, EyeClient *eyeClient, EyeUpdater *eyeUpdater, QWidget *parent) : QWidget(parent) {
    _fieldType = fieldType;
    _eyeClient = eyeClient;
    _eyeUpdater = eyeUpdater;

    // Sets the object name
    this->setObjectName(this->metaObject()->className());

    // Setups the GUI
    setupGUI();
}

MainGUI::~MainGUI() {

}

void MainGUI::setupGUI() {
    // Window config
    this->setWindowTitle("[WREye] MainGUI");
    this->setWindowIcon(QIcon("/usr/share/warthog-robotics/logos/wr_logo.svg"));

    // Set layout
    _windowLayout = new QVBoxLayout();
    this->setLayout(_windowLayout);

    // Create tabs widget
    _windowTabs = new QTabWidget(this);
    _windowTabs->setIconSize(QSize(31, 19));

    // Create WR info bar
    _wrInfo = new QLabel("Warthog Robotics @ USP - São Carlos");
    _wrInfo->setAlignment(Qt::AlignCenter);
    QFont font = _wrInfo->font();
    font.setPointSize(8);
    _wrInfo->setFont(font);

    // Add widgets to layout
    _windowLayout->addWidget(_windowTabs);
    _windowLayout->addWidget(_wrInfo);

    // Create tabs
    _tab_eyeView = new EyeViewGUI(_fieldType, _eyeUpdater);
    _tab_eyeConfig = new EyeConfigGUI(_eyeClient, _eyeUpdater, _tab_eyeView);

    // Add tabs to tabs widget
    _windowTabs->addTab(_tab_eyeView, QIcon(":/fieldAll.png"), "EyeView");
    _windowTabs->addTab(_tab_eyeConfig, QIcon(":/eye.png"), "EyeConfig");
}
