/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#ifndef MULTIOBJECTFILTER_HH
#define MULTIOBJECTFILTER_HH

#include <QMap>
#include "multiobject.hh"
#include <WREye/utils/wrtimer.hh>

#define MOFILTER_MERGE_DISTANCE 0.20f
#define MOFILTER_THRESHOLD_TIME 500 // ms

class MultiObjectFilter {
public:
    MultiObjectFilter();
    ~MultiObjectFilter();

    void reset();
    void addObject(int camId, float confidence, const Position &pos, const Angle &ori = Angle(false, 0));
    QList<MultiObject*> getFiltered();
private:
    // Multi objects map (confidence -> multiobject)
    QMap<float,MultiObject*> _objects;

    // Last known position for avoiding noise
    Position _lastKnownPos;
    WRTimer _noiseFilterTimer;

    // Internal
    QMap<float,MultiObject*> getUnifiedObjects();
    QList<MultiObject*> getUnification(MultiObject *object);
};

#endif // MULTIOBJECTFILTER_HH
