/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/

#ifndef OBJECT_H
#define OBJECT_H

#include <QReadWriteLock>
#include <GEARSystem/gearsystem.hh>
#include <WREye/object/filter/wrfilters.hh>
#include "objectvelocity.hh"

class Object {
public:
    Object(bool enableLossFilter=true, bool enableKalmanFilter=true, bool enableNoiseFilter=true);
    virtual ~Object();

    // Object updater
    void update(double confidence, const Position &pos, const Angle &ori=Angle(false, 0));

    // Object info
    virtual QString name() = 0;
    Position position();
    Velocity velocity();
    Angle orientation();
    double confidence();

    // Set sensor
    void setSensor(Sensor *sensor) { _sensor = sensor; }
protected:
    Sensor* sensor() { return _sensor; }

    // Last info sent to sensor
    Position _position;
    Velocity _velocity;
    Angle _orientation;
private:
    // Update info to sensor
    virtual void updateToSensor() = 0;

    // Internal
    void setUnknown();

    // Confidence info
    double _confidence;

    // Thread-safe mutex
    QReadWriteLock _mutex;

    // Auxiliary flags
    bool _zero;

    // Filters
    LossFilter _lossFilter;
    KalmanFilter2D _kalmanFilter;
    NoiseFilter _noiseFilter;

    // Brute velocity calc
    ObjectVelocity _objVel;

    // Sensor access
    Sensor *_sensor;

    // Noise filter
    bool _unsafe;
};

#endif
