/***
 * Warthog Robotics
 * University of Sao Paulo (USP) at Sao Carlos
 * http://www.warthog.sc.usp.br/
 *
 * This file is part of WREye project.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
#include "robot.hh"
#include <sstream>

QString Robot::name() {
    std::stringstream ss;
    ss << (_color==Colors::BLUE?"Blue":"Yellow") << " robot, id #" << (int)_robotId;
    return QString(ss.str().c_str());
}

Robot::Robot(Colors::Color color, quint8 teamId, quint8 robotId, bool enableLossFilter, bool enableKalmanFilter, bool enableNoiseFilter, bool debugDetection)
    : Object(enableLossFilter, enableKalmanFilter, enableNoiseFilter) {
    _color = color;
    _teamId = teamId;
    _robotId = robotId;
    _debugDetection = debugDetection;
}

Robot::~Robot() {

}

void Robot::updateToSensor() {
    sensor()->setPlayerPosition(_teamId, _robotId, _position);
    sensor()->setPlayerVelocity(_teamId, _robotId, _velocity);
    sensor()->setPlayerOrientation(_teamId, _robotId, _orientation);

    if(_debugDetection)
        std::cout << name().toStdString() << ", set at (" << ((_position.x()>0)?" ":"") << _position.x() << ", " << ((_position.y()>0)?" ":"") << _position.y() << "), velocity=" << _velocity.abs() << ", angle=" << GEARSystem::Angle::toDegrees(_orientation.value()) << ".\n";
}
